#include "Arduino.h"

#include "Thermistor.h"

THERMISTOR::THERMISTOR(uint8_t InputPin, uint8_t PwrPin, uint8_t Type, float P1, float P2, float P3, int VDivRes, uint8_t ResBits, uint8_t Samples, int ESP32ADC, float VSource){
	
	TM_PIN_INPUT = InputPin;
	TM_PIN_PWR = PwrPin;
	TM_TYPE = Type;
	
	switch (TM_TYPE) {
		case 0:
			TM_TNOMINAL = P1;
			TM_RNOMINAL = P2;
			TM_BETA = P3;
			TM_SHC1 = 0;
			TM_SHC2 = 0;
			TM_SHC3 = 0;
			break;

		case 1:
			TM_TNOMINAL = 0;
			TM_RNOMINAL = 0;
			TM_BETA = 0;
			TM_SHC1 = P1;
			TM_SHC2 = P2;
			TM_SHC3 = P3;
			break;
	}
	
	TM_VDIVRES = VDivRes;
	TM_RESBITS = ResBits;
	TM_STEPS = pow(2,TM_RESBITS);
	
	TM_SAMPLES = Samples;
	TM_ESP32ADC = ESP32ADC;
	TM_VSOURCE = VSource;

	pinMode(TM_PIN_INPUT, INPUT);
	pinMode(TM_PIN_PWR, OUTPUT);

}

float THERMISTOR::GetTemp(float *TempF, float *TempC, char *StringF, char *StringC, int *R, int *ADC, float *Volts) {
int i;
int samples[TM_SAMPLES];
float LogR2;
int Stabilize_MS = 250;

  
  // Supply Power to the Thermistor Circuit
  if(TM_PIN_PWR) {
  	digitalWrite(TM_PIN_PWR, HIGH);
  	delay(Stabilize_MS);												// Allow time for Thermistor Power Circuit to stabilize
  }
  
  // take N samples in a row, with a slight delay
  for (i=0; i< TM_SAMPLES; i++) {
   samples[i] = analogRead(TM_PIN_INPUT);
   delay(5);
  }
  
  // Turn Off Power to the Thermistor Circuit
    if(TM_PIN_PWR) digitalWrite(TM_PIN_PWR, LOW);
  
  // average all the samples out
  TM_ADC = 0;
  for (i=0; i< TM_SAMPLES; i++) TM_ADC += samples[i];
  TM_ADC /= TM_SAMPLES;
  
  // Apply ADC Correction for ESP32 non-linear
  if (TM_ESP32ADC) TM_ADC = CorrectESP32ADC(TM_ADC);
  
  // calculate Volts
	TM_VOLTS = (TM_ADC/(TM_STEPS-1)) * TM_VSOURCE;
	
  // Calculate Resistance
	switch (THERMISTOR_ORIENTATION) {
		case 0:			// Thermister on LOW (GND) Side of Voltage Divider
    	TM_RES = TM_VDIVRES /((TM_VSOURCE/TM_VOLTS)-1);	
			break;
		case 1:			// Thermister on HIGH (VCC) Side of Voltage Divider
			TM_RES = ((TM_VSOURCE/TM_VOLTS)-1) * TM_VDIVRES;
			break;
	}
  
  switch (TM_TYPE) {

	  case 0:																			// Calc the BETA Model
	  	TM_TK = TM_RES / TM_RNOMINAL;          		// (R/Ro)
	  	TM_TK = log(TM_TK);                       // ln(R/Ro)
	  	TM_TK /= TM_BETA;                         // 1/B * ln(R/Ro)
	  	TM_TK += 1.0 / (TM_TNOMINAL + 273.15);    // + (1/To)
	  	TM_TK = 1.0 / TM_TK;                      // Invert
	  	break;

	  case 1:																			// Calc the Steinhart Model
	  	LogR2 = log(TM_RES);
	  	TM_TK = (1.0 / (TM_SHC1 + TM_SHC2*LogR2 + TM_SHC3*LogR2*LogR2*LogR2));
	  	break;
  }

  TM_TC = TM_TK - 273.15;
  TM_TF = (TM_TC * 9.0)/ 5.0 + 32.0;

  if (TempF != NULL) *TempF = TM_TF;
  if (TempC != NULL) *TempC = TM_TC;
  
  if (StringF != NULL) dtostrf(TM_TF, 4, 1, StringF);
  if (StringC != NULL) dtostrf(TM_TC, 4, 1, StringC);

  if (R != NULL)   *R = TM_RES;
  if (ADC != NULL) *ADC = TM_ADC;
  if (Volts != NULL) *Volts = TM_VOLTS;
  	
  return TM_TF;
}

float THERMISTOR::CorrectESP32ADC(float ADCin) {
float ADCout;

	// Calculated based on 12-bit resolution (4096 Steps)!
	
	if(TM_STEPS != 4096.0) ADCin = ADCin * (4096.0/TM_STEPS);

	ADCout = (-1.094119555e-04 * sq(ADCin)) + (1.396550444 * ADCin) + 115.0397916;
	
	if (ADCout < 0.0) ADCout = 0.0;
	if (ADCout > 4095.0) ADCout = 4095.0;
		
	if(TM_STEPS != 4096.0) ADCout = ADCout * (TM_STEPS/4096.0);

		
	return ADCout;
}
